from mypy.nodes import CallExpr, RefExpr


def format_json_func(func: str, arg_count: int, arg: str = "x") -> str:
    extra = format_json_trailing_args(func, arg_count)

    return f"{func}({arg}{extra})"


def format_json_trailing_args(func: str, arg_count: int) -> str:
    if func == "json.dumps":
        min_args = 1
        extra = ""

    else:
        min_args = 2
        extra = ", f"

    if arg_count > min_args:
        extra += ", ..."

    return extra


def is_valid_json_call(node: CallExpr) -> bool:
    match node:
        case CallExpr(
            callee=RefExpr(fullname="json.dump" | "json.dumps" as func),
            args=args,
        ):
            min_args = 1 if func == "json.dumps" else 2

            return len(args) >= min_args

    return False



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['refurb_json_minify']

package_data = \
{'': ['*']}

install_requires = \
['refurb>=1.4']

entry_points = \
{'refurb.plugins': ['module = refurb_json_minify']}

setup_kwargs = {
    'name': 'refurb-json-minify',
    'version': '1.0.0',
    'description': 'Detects places where JSON output can be minified',
    'long_description': '# refurb-json-minify\n\nA small plugin for [Refurb](https://github.com/dosisod/refurb) aimed at minifying JSON outputs.\n\n## Why is this important?\n\nJSON is a widely used format for data exchange, whether that be APIs talking over the\ninternet, metadata being stored in a database, or config files stored on a user\'s\nfilesystem. Although CPU and harddrive space is getting cheaper and cheaper, it isn\'t\nfree, and being mindful of resources can lead to faster and more efficient programs.\n\n## Supported Checks\n\n### `JMIN100`: Use `separators`\n\nThe [`json.dump`](https://docs.python.org/3/library/json.html#json.dump) and\n[`json.dumps`](https://docs.python.org/3/library/json.html#json.dumps) functions\nallow for an optional `separators` field which specifies what characters to use\nfor colons (`:`) and commas (`,`) in the JSON output. Normally there is whitespace\nafter these characters, but you can change this to use a more compact format.\n\nHere is a simple example comparing the output of `json.dumps()` with and without\n`separators` specified:\n\n```python\nimport json\n\ndata = {\n  "hello": "world",\n  "numbers": [1, 2, 3, 4],\n}\n\na = json.dumps(data)\nb = json.dumps(data, separators=(",", ":"))\n\nprint(f"{len(a)=}", f"{len(b)=}")\n```\n\nWhen we run this, we get:\n\n```\nlen(a)=43 len(b)=37\n```\n\nBy reducing the whitespace in our JSON output we where able to shave off 6 bytes, or about\n16% in this example.\n\n### `JMIN101`: Don\'t `json.dump()` integers\n\nDon\'t call `json.dump()` on integers, use `str()` instead since they share the\nsame representation.\n',
    'author': 'dosisod',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/dosisod/refurb-json-minify',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
}


setup(**setup_kwargs)



from dataclasses import dataclass

from mypy.nodes import CallExpr, IntExpr, RefExpr, Var

from refurb.error import Error
from refurb_json_minify.common import format_json_func, is_valid_json_call


@dataclass
class ErrorInfo(Error):
    """
    Don't dump integers to JSON values, just convert them to strings since
    they use the same representation.

    Bad:

    ```
    data = json.dumps(123)
    ```

    Good:

    ```
    data = str(123)
    ```
    """

    name = "dont-dump-integers"
    prefix = "JMIN"
    code = 101


def check(node: CallExpr, errors: list[Error]) -> None:
    match node:
        case CallExpr(
            callee=RefExpr(fullname=func),
            args=[arg, *_] as args,
        ) if is_valid_json_call(node):
            match arg:
                case IntExpr(value=value):
                    expr = str(value)
                    replace = f'"{value}"'

                case RefExpr(node=Var(type=ty)) if str(ty) == "builtins.int":
                    expr = (
                        arg.name
                        if hasattr(arg, "name")
                        else arg.fullname or ""
                    )

                    replace = f"str({expr})"

                case _:
                    return

            if func == "json.dump":
                replace = f"f.write({replace})"

            old = format_json_func(func, len(args), arg=expr)

            errors.append(
                ErrorInfo(
                    node.line,
                    node.column,
                    f"Replace `{old}` with `{replace}`",
                )
            )



from dataclasses import dataclass

from mypy.nodes import CallExpr, NameExpr, RefExpr

from refurb.error import Error
from refurb_json_minify.common import (
    format_json_trailing_args,
    is_valid_json_call,
)


@dataclass
class ErrorInfo(Error):
    """
    By default when calling `json.dumps()` Python will emit spaces around
    colons and commas. By removing these spaces you can compress your JSON
    output slightly, allowing you will store less JSON in your database or
    filesystem, send less bytes to clients using your API, and much more.

    This does not effect reading of the JSON: Any JSON-compliant library
    should be able to parse the minified data without any issues.

    Bad:

    ```
    x = json.dumps({"nums": [1, 2, 3, 4, 5]})

    print(x)  # {"nums": [1, 2, 3, 4, 5]}
    ```

    Good:

    ```
    x = json.dumps({"nums": [1, 2, 3, 4, 5]}, separators=(",", ":"))

    print(x)  # {"nums":[1,2,3,4,5]}
    ```
    """

    name = "use-json-separators"
    prefix = "JMIN"
    code = 100


def check(node: CallExpr, errors: list[Error]) -> None:
    match node:
        case CallExpr(
            callee=RefExpr(fullname=func),
            args=args,
            arg_names=arg_names,
        ) if is_valid_json_call(node):
            for i, name in enumerate(arg_names):
                if name == "separators":
                    return

                if name == "indent":
                    match args[i]:
                        case NameExpr(fullname="builtins.None"):
                            pass

                        case _:
                            return

            extra = format_json_trailing_args(func, len(args))

            separators = ', separators=(",", ":")'

            errors.append(
                ErrorInfo(
                    node.line,
                    node.column,
                    f"Replace `{func}(x{extra})` with `{func}(x{extra}{separators})`",  # noqa: E501
                )
            )
